<?php

declare(strict_types=1);

namespace Dedi\SyliusDpdPickUpPlugin\Service;

use Dedi\SyliusDpdPickUpPlugin\Entity\OrderInterface;
use Dedi\SyliusDpdPickUpPlugin\Entity\ShippingMethodInterface;
use Dedi\SyliusDpdPickUpPlugin\Exception\MissingPudoException;
use Dedi\SyliusDpdPickUpPlugin\Model\Pudo;
use Dedi\SyliusDpdPickUpPlugin\Specification\DoesAddressRequireProvinceCodeSpecification;
use Sylius\Component\Core\Model\Address;
use Sylius\Component\Core\Model\ShipmentInterface;

class PudoSelector
{
    private readonly DoesAddressRequireProvinceCodeSpecification $requireProvinceCodeSpecification;
    private readonly PudoToProvinceCodeConverterInterface $pudoToProvinceCodeConverter;

    public function __construct(
        DoesAddressRequireProvinceCodeSpecification $requireProvinceCodeSpecification,
        PudoToProvinceCodeConverterInterface $pudoToProvinceCodeConverter,
    ) {
        $this->requireProvinceCodeSpecification = $requireProvinceCodeSpecification;
        $this->pudoToProvinceCodeConverter = $pudoToProvinceCodeConverter;
    }

    /**
     * Update order to set the selected pudo if any.
     *
     * @throw MissingPudoException
     */
    public function __invoke(
        OrderInterface $order,
        ?Pudo $pudo,
    ): void {
        /** @var ShipmentInterface $shippment */
        $shippment = $order->getShipments()->first();

        /** @var ShippingMethodInterface $method */
        $method = $shippment->getMethod();

        if (null === $pudo && $method->isDpdPickUp()) {
            throw new MissingPudoException();
        }

        if (null === $pudo || !$method->isDpdPickUp()) {
            $order->setPudoId(null);

            return;
        }

        /** @var Address $address */
        $address = $order->getShippingAddress();
        $address->setCompany($pudo->name);
        $address->setCity($pudo->city);
        $address->setPostcode($pudo->zipCode);
        $address->setProvinceCode(
            $this->requireProvinceCodeSpecification->__invoke($address) ?
                $this->pudoToProvinceCodeConverter->transform($pudo) : null,
        );
        $address->setProvinceName(null);

        $street = $pudo->address1;
        if (null !== $pudo->address2 && '' !== $pudo->address2) {
            $street .= sprintf(', %s', $pudo->address2);
        }
        if (null !== $pudo->address3 && '' !== $pudo->address3) {
            $street .= sprintf(', %s', $pudo->address3);
        }
        $address->setStreet($street);

        $order->setPudoId($pudo->id);
    }
}
